/*
   ----------------------------------------------------------------
   Description:

      This class reads and writes files in vcard-format. This is only 
      in a simple form. Grouping vcards is not supported.
   
   
   Programmers:
   
      Bernd Hohenester, newsletter@hobe.de
   
   
   History:
   
      12 Nov 2005  initial release;
                   coded while dBKON 2005 in dZenbach, Germany
                   http://www.dbkon.org
                   
   
   Usage:
   
      set procedure to vcard.cc additive
      v = new vcard()
      v.Name = "Hohenester"
      v.FirstName = "Bernd"
      v.Email = "newsletter@hobe.de"
      v.Url = "http://www.hobe.de"
      v.write()
      release object v
      v = null
      close procedure vcard.cc
      
      
   Properties:

   read(cReadFile) - reads a vcard-file
                     The parameter cReadFile is optional. When empty,
                     the file-dialog comes up and asks for the .vcf
                     
   write(cWriteFile) - writes a vcard-file
                       The parameter cWriteFile is optional. When empty,
                       the file-dialog comes up and asks for the filename.
                       
      
*/


class Vcard of object custom
  
  this.Version = "2.1"
  
  this.Name = ""
  this.FirstName = ""
  this.Additional = ""
  this.Prefix = ""
  this.Title = ""
  
  this.POBox = ""
  this.Extended = ""
  this.Street = ""
  this.City = ""
  this.Region = ""
  this.ZIP = ""
  this.Country = ""
  
  this.Birthday = ""
  this.Occupation = ""
  
  this.Private = ""
  this.Work = ""
  this.Fax = ""
  this.Cellular = ""
  this.Email = ""
  this.URL = ""
  
  this.Note = ""
  
  
  function read(cReadFile)
    
    if empty(cReadFile)
      try
        cReadFile = getfile("*.vcf", "Choose vcard", true, "vcf")
      catch (exception e)
        cReadFile = getfile("*.vcf", "Choose vcard", true)
      endtry
      
      if empty(cReadFile)
        return false
      endif
    endif
    
    
    local lRueck, f, cTemp, lFoundBegin, lFoundEnd
    
    f = new file()
    if f.exists(cReadFile)
      f.open(cReadFile, "R")
      
      f.seek(0, 0)
      
      lFoundBegin = false
      lFoundEnd = false
      lContinueNote = false
      
      do
        cTemp = f.gets()
        
        if "BEGIN:VCARD" $ cTemp.toUpperCase()
          lFoundBegin = true
        endif
        
        if lFoundBegin
          do case
              
              // N:FamilyName;GivenName;Additional;Prefix;Suffix
            case cTemp.toUpperCase().left(2) == "N:"
              lContinueNote = false
              cTemp = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)  // N: abschneiden
              
              oArray = new array(5)
              for i = 1 to oArray.size
                oArray[i] = ""
              endfor
              class::SplitLine(cTemp, oArray)
              
              this.Name = oArray[1]
              this.FirstName = oArray[2]
              this.Additional = oArray[3]
              this.Prefix = oArray[4]
              this.Title = oArray[5]
              
              
              // ADR:POBox;Extended;Street;City;Region;ZIP;Country
            case cTemp.toUpperCase().left(3) == "ADR"
              lContinueNote = false
              cTemp = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              
              oArray = new array(7)
              for i = 1 to oArray.size
                oArray[i] = ""
              endfor
              class::SplitLine(cTemp, oArray)
              
              this.POBox = oArray[1]
              this.Extended = oArray[2]
              this.Street = oArray[3]
              this.City = oArray[4]
              this.Region = oArray[5]
              this.ZIP = oArray[6]
              this.Country = oArray[7]
              
              
              // BDAY:
            case cTemp.toUpperCase().left(5) == "BDAY:"
              lContinueNote = false
              cTemp = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              if "-" $ cTemp
                this.Birthday = "" + new date(cTemp)
              else
                this.Birthday = "" + new date(cTemp.substring(0, 4) + "-" + cTemp.substring(4, 6) + "-" + cTemp.substring(6, 8)        )
              endif
              
              
              // ROLE:
            case cTemp.toUpperCase().left(5) == "ROLE:"
              lContinueNote = false
              this.Occupation = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              
              
              // TEL
            case cTemp.toUpperCase().left(3) == "TEL"
              lContinueNote = false
              
              if "HOME" $ cTemp.toUpperCase()
                this.Private = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              endif
              
              if "WORK" $ cTemp.toUpperCase()
                this.Work = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              endif
              
              if "FAX" $ cTemp.toUpperCase()
                this.Fax = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              endif
              
              if "CELL" $ cTemp.toUpperCase()
                this.Cellular = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              endif
              
              
              // EMAIL:
            case cTemp.toUpperCase().left(6) == "EMAIL:"
              lContinueNote = false
              this.Email = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              
              
              // URL:
            case cTemp.toUpperCase().left(4) == "URL:"
              lContinueNote = false
              this.URL = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              
              
              // NOTE:
            case cTemp.toUpperCase().left(4) == "NOTE"
              this.Note = cTemp.substring(cTemp.indexOf(":") + 1, cTemp.length)
              lContinueNote = true
              
              // FN:
            case cTemp.toUpperCase().left(3) == "FN:"
              lContinueNote = false
              
              // TZ:
            case cTemp.toUpperCase().left(3) == "TZ:"
              lContinueNote = false
              
              // REV:
            case cTemp.toUpperCase().left(4) == "REV:"
              lContinueNote = false
              
              
            otherwise
              if lContinueNote
                this.Note += chr(13) + chr(10) + cTemp
              endif
              
              
          endcase
          
          if "END:VCARD" $ cTemp.toUpperCase()
            lFoundEnd = true
          endif
          
        endif
        
      until f.eof()
      
      f.close()
      
      lRueck = true
      
      if not lFoundBegin or not lFoundEnd
        msgbox("The file " + cReadFile + " is not a valid vcard-file!", "Error", 48)
        lRueck = false
      endif
      
    else
      msgbox("The vcard-file " + cReadFile + " does not exist!", "Error", 48)
      lRueck = false
    endif
  return lRueck
  
  
  
  
  function write(cWriteFile)
    
    if empty(cWriteFile)
      try
        cWriteFile = putfile("Save vcard to", this.Name.righttrim() + ".vcf", "vcf", true, "vcf")
      catch (exception e)
        cWriteFile = putfile("Save vcard to", this.Name.righttrim() + ".vcf", "vcf", true)
      endtry
      
      if empty(cWriteFile)
        return
      endif
    endif
    
    cWriteFile = class::ValidFileName(cWriteFile)
    
    local f, d, x, hh, mm
    
    f = new file()
    f.create(cWriteFile, "W")
    
    f.puts("BEGIN:VCARD")
    f.puts("VERSION:" + this.Version)
    f.puts("N:" + this.Name.righttrim() + ";" + this.FirstName.righttrim() + ";" + this.Additional.righttrim() + ";" + this.Prefix.righttrim() + ";" + this.Title.righttrim())
    f.puts("FN:" + (this.Prefix.righttrim() + " ").lefttrim() + (this.Title.righttrim() + " " + this.FirstName.righttrim() + " ").lefttrim() + (this.Additional.righttrim() + " " + this.Name.righttrim()).lefttrim())
    
    if not empty(this.POBox) ;
        or not empty(this.Extended) ;
        or not empty(this.Street) ;
        or not empty(this.City) ;
        or not empty(this.Region) ;
        or not empty(this.ZIP) ;
        or not empty(this.Country)
      f.puts("ADR;HOME:" + this.POBox.righttrim() + ";" + this.Extended.righttrim() + ";" + this.Street.righttrim() + ";" + this.City.righttrim() + ";" + this.Region.righttrim() + ";" + this.ZIP.righttrim() + ";" + this.Country.righttrim())
      f.puts("LABEL;HOME;ENCODING=QUOTED-PRINTABLE:" + class::String2QP(this.Street.righttrim() + ", " + (this.ZIP.righttrim() + " ").lefttrim() + this.City.righttrim() + iif(not empty(this.Country), ", " + this.Country.righttrim(), "")))
    endif
    
    if not empty(this.Birthday)
      d = new date(this.Birthday)
      f.puts("BDAY:" + transform(d.year, "9999") + transform(d.month + 1,"@L 99") + transform(d.date, "@L 99"))
    endif
    
    if not empty(this.Occupation)
      f.puts("ROLE:" + this.Occupation.rightTrim())
    endif
    
    if not empty(this.Private)
      f.puts("TEL;HOME:" + class::canonical(this.Private))
    endif
    
    if not empty(this.Work)
      f.puts("TEL;WORK:" + class::canonical(this.Work))
    endif
    
    if not empty(this.Fax)
      f.puts("TEL;FAX:" + class::canonical(this.Fax))
    endif
    
    if not empty(this.Cellular)
      f.puts("TEL;PREF;CELL:" + class::canonical(this.Cellular))
    endif
    
    if not empty(this.Email)
      f.puts("EMAIL:" + this.Email.righttrim())
    endif
    
    if not empty(this.URL)
      f.puts("URL:" + this.URL.righttrim())
    endif
    
    if not empty(this.Note)
      f.puts("NOTE:" + this.Note.righttrim())
    endif
    
    x = new date().getTimeZoneOffset()
    if x >= 0
      hh = transform(int(x / 60), "@L +99")
      mm = transform(mod(x, 60), "@L 99")
    else
      hh = transform(int(x / 60), "@L 999")
      mm = transform(mod(x, 60), "@L 99")
    endif
    f.puts("TZ:" + hh + mm)
    
/*
    d = new date()
    f.puts("REV:" + ;
      d.year + ;
      transform(d.month + 1, "@L 99") + ;
      transform(d.date, "@L 99") + ;
      "T" + ;
      transform(d.hour, "@L 99") + ;
      transform(d.minute, "@L 99") + ;
      transform(d.second, "@L 99"))
*/

    f.puts("END:VCARD")
    
    f.flush()
    f.close()
    
  return
  
  
  
  function ValidFileName(cFileName)
    
    local cTemp, cRueck, i, ch
    
    cTemp = cFileName.righttrim().lefttrim()
    cRueck = ""
    
    for i = 1 to cTemp.length
      ch = substr(cTemp, i, 1)
      
      do case
        case ch $ '/*?"<>|'
          
        otherwise
          cRueck += ch
      endcase
    endfor
  return (cRueck)
  
  
  
  function canonical(cNummer)
    
    local cRueck, cPattern, re
    
    cRueck = cNummer.lefttrim().righttrim()
    
    cPattern = "(\+\d{2,4}|0)\s?(\d{2,4})\s?(\d{2,7})(-| )?(\d{4})?"
    re = new oleAutoClient("VBScript.RegExp")
    re.pattern = cPattern
    
    cRueck = re.replace( cNummer , "$1-$2-$3$4$5")
    
    if substr(cRueck, 1, 2) == "0-"
      cRueck = "+49-" + substr(cRueck, 3)
    endif
    
    re = null
  return (cRueck.lefttrim().righttrim())
  
  
  
  function SplitLine(cLine, oArray)
    
    local nIndexZaehler, cTemp, i, ch
    
    cLine = cLine.righttrim().lefttrim()
    nIndexZaehler = 1
    cTemp = ""
    
    for i = 1 to cLine.length
      ch = substr(cLine, i, 1)
      
      if ch == ";"
        oArray[nIndexZaehler] = cTemp
        nIndexZaehler += 1
        cTemp = ""
      else
        cTemp += ch
      endif
      
    endfor
    
    oArray[nIndexZaehler] = cTemp
    
  return
  
  
  function String2QP(cLine)
    
    local cTemp, i, ch
    
    cLine = cLine.righttrim().lefttrim()
    cTemp = ""
    
    for i = 1 to cLine.length
      ch = substr(cLine, i, 1)
      
      do case
        case (asc(ch) >= 33 and asc(ch) <= 60) or (asc(ch) >= 62 and asc(ch) <= 126)
          cTemp += ch
        case ch == "="
          cTemp += "=3D"
        otherwise
          cTemp += "=" + itoh(asc(ch))
      endcase
    endfor
    
    
    // Zeilenlnge max. 75 Zeichen einschl. =
    #define CRLF chr(13)+chr(10)
    
    if cTemp.length > 75
      cLine = cTemp
      cTemp = ""
      
      do while cLine.length > 75
        
        do case
          case substr(cLine, 74, 1) == "="
            cTemp += substr(cLine, 1, 73) + "=" + CRLF
            cLine = substr(cLine, 74)
          case substr(cLine, 75, 1) == "="
            cTemp += substr(cLine, 1, 74) + "=" + CRLF
            cLine = substr(cLine, 75)
          otherwise
            cTemp += substr(cLine, 1, 75) + "=" + CRLF
            cLine = substr(cLine, 76)
        endcase
        
      enddo
      cTemp += cLine  // Rest anhngen
    endif
    
  return (cTemp)
  
  
endclass
